@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2005-2018 Intel Corporation
+    Copyright (c) 2005-2019 Intel Corporation
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -24,10 +24,6 @@
 namespace tbb {
     namespace internal {
 
-        //
-        // This is not under __TBB_ITT_STRUCTURE_API because these values are used directly in flow_graph.h.
-        //
-
         // include list of index names
         #define TBB_STRING_RESOURCE(index_name,str) index_name,
         enum string_index {
@@ -138,8 +134,7 @@ namespace tbb {
         void __TBB_EXPORTED_FUNC itt_store_pointer_with_release_v3(void *dst, void *src);
         void* __TBB_EXPORTED_FUNC itt_load_pointer_with_acquire_v3(const void *src);
         void* __TBB_EXPORTED_FUNC itt_load_pointer_v3( const void* src );
-#if __TBB_ITT_STRUCTURE_API
-        enum itt_domain_enum { ITT_DOMAIN_FLOW=0 };
+        enum itt_domain_enum { ITT_DOMAIN_FLOW=0, ITT_DOMAIN_MAIN=1, ITT_DOMAIN_ALGO=2, ITT_NUM_DOMAINS };
 
         void __TBB_EXPORTED_FUNC itt_make_task_group_v7( itt_domain_enum domain, void *group, unsigned long long group_extra,
                                                          void *parent, unsigned long long parent_extra, string_index name_index );
@@ -154,7 +149,6 @@ namespace tbb {
         void __TBB_EXPORTED_FUNC itt_region_begin_v9( itt_domain_enum domain, void *region, unsigned long long region_extra,
                                                       void *parent, unsigned long long parent_extra, string_index name_index );
         void __TBB_EXPORTED_FUNC itt_region_end_v9( itt_domain_enum domain, void *region, unsigned long long region_extra );
-#endif // __TBB_ITT_STRUCTURE_API
 
         // two template arguments are to workaround /Wp64 warning with tbb::atomic specialized for unsigned type
         template <typename T, typename U>
@@ -238,12 +232,6 @@ namespace tbb {
             call_itt_notify_v5((int)t, ptr);
         }
 
-#else
-        inline void call_itt_notify(notify_type /*t*/, void * /*ptr*/) {}
-
-#endif // TBB_USE_THREADING_TOOLS
-
-#if __TBB_ITT_STRUCTURE_API
         inline void itt_make_task_group( itt_domain_enum domain, void *group, unsigned long long group_extra,
                                          void *parent, unsigned long long parent_extra, string_index name_index ) {
             itt_make_task_group_v7( domain, group, group_extra, parent, parent_extra, name_index );
@@ -276,9 +264,81 @@ namespace tbb {
         inline void itt_region_end( itt_domain_enum domain, void *region, unsigned long long region_extra  ) {
             itt_region_end_v9( domain, region, region_extra );
         }
-#endif // __TBB_ITT_STRUCTURE_API
+#else
+        inline void call_itt_notify(notify_type /*t*/, void* /*ptr*/) {}
+
+        inline void itt_make_task_group( itt_domain_enum /*domain*/, void* /*group*/, unsigned long long /*group_extra*/,
+                                         void* /*parent*/, unsigned long long /*parent_extra*/, string_index /*name_index*/ ) {}
+
+        inline void itt_metadata_str_add( itt_domain_enum /*domain*/, void* /*addr*/, unsigned long long /*addr_extra*/,
+                                          string_index /*key*/, const char* /*value*/ ) {}
+
+        inline void itt_relation_add( itt_domain_enum /*domain*/, void* /*addr0*/, unsigned long long /*addr0_extra*/,
+                                      itt_relation /*relation*/, void* /*addr1*/, unsigned long long /*addr1_extra*/ ) {}
+
+        inline void itt_task_begin( itt_domain_enum /*domain*/, void* /*task*/, unsigned long long /*task_extra*/,
+                                    void* /*parent*/, unsigned long long /*parent_extra*/, string_index /*name_index*/ ) {}
+
+        inline void itt_task_end( itt_domain_enum /*domain*/ ) {}
+
+        inline void itt_region_begin( itt_domain_enum /*domain*/, void* /*region*/, unsigned long long /*region_extra*/,
+                                      void* /*parent*/, unsigned long long /*parent_extra*/, string_index /*name_index*/ ) {}
+
+        inline void itt_region_end( itt_domain_enum /*domain*/, void* /*region*/, unsigned long long /*region_extra*/ ) {}
+#endif // TBB_USE_THREADING_TOOLS
 
     } // namespace internal
 } // namespace tbb
 
+#if TBB_PREVIEW_FLOW_GRAPH_TRACE
+#include <string>
+
+namespace tbb {
+namespace profiling {
+namespace interface10 {
+
+#if TBB_USE_THREADING_TOOLS && !(TBB_USE_THREADING_TOOLS == 2)
+class event {
+/** This class supports user event traces through itt.
+    Common use-case is tagging data flow graph tasks (data-id)
+    and visualization by Intel Advisor Flow Graph Analyzer (FGA)  **/
+//  TODO: Replace implementation by itt user event api.
+
+    const std::string my_name;
+
+    static void emit_trace(const std::string &input) {
+        itt_metadata_str_add( tbb::internal::ITT_DOMAIN_FLOW, NULL, tbb::internal::FLOW_NULL, tbb::internal::USER_EVENT, ( "FGA::DATAID::" + input ).c_str() );
+    }
+
+public:
+    event(const std::string &input)
+              : my_name( input )
+    { }
+
+    void emit() {
+        emit_trace(my_name);
+    }
+
+    static void emit(const std::string &description) {
+        emit_trace(description);
+    }
+
+};
+#else // TBB_USE_THREADING_TOOLS && !(TBB_USE_THREADING_TOOLS == 2)
+// Using empty struct if user event tracing is disabled:
+struct event {
+    event(const std::string &) { }
+
+    void emit() { }
+
+    static void emit(const std::string &) { }
+};
+#endif // TBB_USE_THREADING_TOOLS && !(TBB_USE_THREADING_TOOLS == 2)
+
+} // interfaceX
+using interface10::event;
+} // namespace profiling
+} // namespace tbb
+#endif // TBB_PREVIEW_FLOW_GRAPH_TRACE
+
 #endif /* __TBB_profiling_H */
